@@ -61,7 +61,7 @@ if IN_COVERAGE_TESTS:
         if key in written:
             return
         written.add(key)
-        with open("branch-check", "a") as log:
+        with open("branch-check", mode="a", encoding="utf-8") as log:
             log.write(json.dumps({"name": name, "value": value}) + "\n")
 
     description_stack = []
@@ -99,7 +99,6 @@ if IN_COVERAGE_TESTS:
 
         return accept
 
-
 else:  # pragma: no cover
 
     def check_function(f: Func) -> Func:
